@@ -320,25 +320,30 @@ def RunturbESN(esn, u_train: Union[np.ndarray, torch.Tensor]=None,
 
     mse_test = ComputeMSE(y_test = esn.y_test,y_pred = y_pred_test)
     
-    #----------------------------------------------------------
-    #4. (optional) Validation Phase (for now only in auto mode)
-    #----------------------------------------------------------
+    #-------------------------------------------------------------------------
+    #4. (optional) Validation Phase (for now only in auto & teacherforce mode)
+    #-------------------------------------------------------------------------
     mse_val = None
+    y_pred_val = None
 
-    if esn.mode == _ESN_MODES[0]:
-        if None not in [esn.u_val, esn.y_val]:
+    
+    if None not in [esn.u_val, esn.y_val]:
 
-            validationLength = esn.y_val.shape[0]
-            if u_pre_val is not None:
-                X_pre_val = esn.propagate(u = u_pre_val, transientTime = u_pre_val.shape[0]-1)
-                #esn.val_init_input = u_pre_val[-1,:].reshape(1,esn.n_input)   #this is already set in check_user_input
-            else:
-                # validation phase directly preceeded by testing phase
-                X_pre_val = esn.x_pred
-                esn.val_init_input = esn.y_test[-1,:].reshape(1,esn.n_input)
+        validationLength = esn.y_val.shape[0]
+        if u_pre_val is not None:
+            X_pre_val = esn.propagate(u = u_pre_val, transientTime = u_pre_val.shape[0]-1)
+            #esn.val_init_input = u_pre_val[-1,:].reshape(1,esn.n_input)   #this is already set in check_user_input
+        else:
+            # validation phase directly preceeded by testing phase
+            X_pre_val = esn.x_pred
+            esn.val_init_input = esn.y_test[-1,:].reshape(1,esn.n_input)
 
+        if esn.mode == _ESN_MODES[0]:
             y_pred_val, esn.x_val = esn.predict(X=X_pre_val, testingLength=validationLength,init_input=esn.val_init_input)
-            mse_val = ComputeMSE(y_test = esn.y_val,y_pred = y_pred_val)
+        if esn.mode == _ESN_MODES[1]:
+            y_pred_val, esn.x_val = esn.teacherforce(X = esn.x_fit, testingLength = esn.validationLength,u=esn.u_val)
+
+        mse_val = ComputeMSE(y_test = esn.y_val,y_pred = y_pred_val)
         
 
     return mse_train, mse_test, mse_val, y_pred_test, y_pred_val
